--[[
    Database of all minipets and mounts
    $Revision: 243 $
]]--

--[[
Copyright (c) 2008-2011, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local MAJOR_VERSION = "LibLordFarlander-PetAndMountDatabase-1.1"
local MINOR_VERSION = tonumber( ( "$Revision: 243 $" ):match("(%d+)" ) ) + 90000

local LibPetAndMountDatabase, oldminor = LibStub:NewLibrary( MAJOR_VERSION, MINOR_VERSION )
if( not LibPetAndMountDatabase ) then
    return
end--if

if( not LibPetAndMountDatabase.FoundSets ) then
    LibPetAndMountDatabase.FoundSets = {}
end--if
if( not LibPetAndMountDatabase.SubSets ) then
    LibPetAndMountDatabase.SubSets = {}
end--if

local LibPeriodicTable = LibStub( "LibPeriodicTable-3.1", true )
if( not LibPeriodicTable ) then
    error( MAJOR_VERSION .. " requires LibPeriodicTable-3.1" )
end--if

local TR = LibStub( "LibLordFarlander-TableRecycler-1.0", true )
if( not TR ) then
    error( MAJOR_VERSION .. " requires LibLordFarlander-TableRecycler-1.0" )
    return
end--if

--[[
    Negative number is a SPELL instead of an ITEM
]]--

function LibPetAndMountDatabase.NormalizeSetName( set )
    if( set:find( "PetAndMountDatabase." ) ~= 1 ) then
        set = "LibLordFarlander-PetAndMountDatabase." .. set
    end--if
    return set
end--LibPetAndMountDatabase.NormalizeSetName( set )

function LibPetAndMountDatabase:HasSet( set, normalize )
    local sets = LibPeriodicTable.sets

    if( ( normalize == true ) or ( normalize == nil ) ) then
        set = self.NormalizeSetName( set )
    end--if
    if( self.FoundSets[set] ) then
        return true
    end--if
    for setName, _ in pairs( sets ) do
        if( setName:find( set ) == 1 ) then
            self.FoundSets[set] = true
            return true
        end--if
    end--if
    return false
end--LibPetAndMountDatabase:HasSet( set, normalize )

function LibPetAndMountDatabase:GetSubsets( set, normalize )
    local sets = LibPeriodicTable.sets

    if( ( normalize == true ) or ( normalize == nil ) ) then
        set = self.NormalizeSetName( set )
    end--if
    if( self.SubSets[set] ) then
        return self.SubSets[set]
    end--if
    set = set .. "."
    for setName, _ in pairs( sets ) do
        local start, ends = setName:find( set )

        if( ( start == 1 ) and ( ends ~= setName:len() ) ) then
            setName = setName:sub( ends + 1 )
            if( setName:find( "." ) ) then
                setName = ("%s%s"):format( set, strsplit( ".", setName ) )
            end--if
            if( self.SubSets[set] == nil ) then
                self.SubSets[set] = {}
            end--if
            self.SubSets[set][setName] = setName
        end--if
    end--if
    return self.SubSets[set]
end--LibPetAndMountDatabase:GetSubsets( set, normalize )

function LibPetAndMountDatabase:ItemInSet( item, set, normalize )
    if( item == nil ) then
        return false
    end--if
    if( ( normalize == true ) or ( normalize == nil ) ) then
        set = self.NormalizeSetName( set )
    end--if
    return LibPeriodicTable:ItemInSet( item, set )
end--LibPetAndMountDatabase:ItemInSet( item, set, normalize )

function LibPetAndMountDatabase:SpellInSet( spellID, set, normalize )
    if( spellID == nil ) then
        return false
    end--if
    if( spellID > 0 ) then
        spellID = spellID * -1
    end--if
    return self:ItemInSet( spellID, set, normalize )
end--LibPetAndMountDatabase:SpellInSet( item, set )

function LibPetAndMountDatabase:IterateSet( set, normalize )
    if( ( normalize == true ) or ( normalize == nil ) ) then
        set = self.NormalizeSetName( set, normalize )
    end--if
    return LibPeriodicTable:IterateSet( set )
end--LibPetAndMountDatabase:IterateSet( set, normalize )

function LibPetAndMountDatabase.GetConditions( value )
    local conditions = value:match( "%[(.-)%]" )
    local returnval

    if( conditions ) then
        conditions = TR:GetTableWithInserts( strsplit( ";", conditions ) )
        for _, condition in pairs( conditions ) do
            local name, value = condition:match( "(.-)=(.+)" )

            if( name and value ) then
                if( not returnval ) then
                    returnval = TR:GetTable()
                end--if
                returnval[name] = value
            end--if
        end--for
        TR:ReturnTable( conditions )
    end--if
    return returnval
end--if

function LibPetAndMountDatabase.GetMountAttributes( value, skipskills )
    if( value == LibPetAndMountDatabase ) then
        -- backwards compatibility
        value = value2
        skipskills = false
    end--if

    local groundSpeed, aquaticSpeed, flySpeed, passengers, skills = strsplit( "|", value )
    local swimSpeed

    if( skills and not( skipskills ) ) then
        local skillString = skills

        skills = TR:GetTable()
        skillList = TR:GetTableWithInserts( strsplit( ";", skillString ) )
        for _, skill in pairs( skillList ) do
            local skillName, value = skill:sub( 1, 1 ), tonumber( skill:sub( 2 ) )

            if( skillName == "R" ) then
                skillName = "Riding"
            elseif( skillName == "E" ) then
                skillName = "Engineering"
            elseif( skillName == "T" ) then
                skillName = "Tailoring"
            elseif( skillName == "N" ) then
                skillName = "Enchanting"
            elseif( skillName == "B" ) then
                skillName = "Blacksmithing"
            elseif( skillName == "A" ) then
                skillName = "Alchemy"
            elseif( skillName == "J" ) then
                skillName = "Jewelcrafting"
            elseif( skillName == "M" ) then
                skillName = "Mining"
            elseif( skillName == "H" ) then
                skillName = "Herbalism"
            elseif( skillName == "L" ) then
                skillName = "Leatherworking"
            elseif( skillName == "S" ) then
                skillName = "Skinning"
            elseif( skillName == "I" ) then
                skillName = "Inscription"
            elseif( skillName == "C" ) then
                skillName = "Cooking"
            elseif( skillName == "+" ) then
                skillName = "First Aid"
            elseif( skillName == "F" ) then
                skillName = "Fishing"
            elseif( skillName == "Y" ) then
                skillName = "Archeology"
            end--if
            skills[skillName] = value
        end--if
    end--if

    aquaticSpeed, swimSpeed = strsplit( ";", aquaticSpeed and aquaticSpeed or "" )

    groundSpeed = groundSpeed and tonumber( groundSpeed )
    aquaticSpeed = aquaticSpeed and tonumber( aquaticSpeed )
    flySpeed = flySpeed and tonumber( flySpeed )
    swimSpeed = swimSpeed and tonumber( swimSpeed )

    local usegroundSpeed = groundSpeed and not( groundSpeed == -1 )
    local useaquaticSpeed = aquaticSpeed and not( aquaticSpeed == -1 )
    local useflySpeed = flySpeed and not( flySpeed == -1 )
    local useswimSpeed = swimSpeed and not( swimSpeed == -1 )

    return groundSpeed, aquaticSpeed, flySpeed, passengers and tonumber( passengers ) or 1, skills,
        swimSpeed, not( usegroundSpeed ) and not( useaquaticSpeed ) and not( useflySpeed ) and not( useswimSpeed )
end--LibPetAndMountDatabase,GetMountAttributes( value )

function LibPetAndMountDatabase:GetPlayersCompanions( ctype )
    local returnList = TR:GetTable()

    for i = 1, GetNumCompanions( ctype ) do
        local creatureID, creatureName, creatureSpellID, creatureIcon, _, mountType = GetCompanionInfo( ctype, i )

        returnList[i] =
            TR:GetTable( "spellID", creatureSpellID,
                         "icon", creatureIcon,
                         "modelID", creatureID,
                         "name", creatureName )
        if( mountType ) then
            returnList[i].ground = bit.band( mountType, 0x01 ) ~= 0
            returnList[i].flying = bit.band( mountType, 0x02 ) ~= 0
            returnList[i].swimming = bit.band( mountType, 0x04 ) ~= 0
            returnList[i].underwater = bit.band( mountType, 0x08 ) ~= 0
            returnList[i].canJump =  bit.band( mountType, 0x10 ) ~= 0
        end--if
    end--for
    return returnList
end--LibPetAndMountDatabase:GetPlayersCompanions( ctype )

function LibPetAndMountDatabase:IsCompanionType( companionName, ctype )
    for i = 1, GetNumCompanions( ctype ) do
        if( select( 2, GetCompanionInfo( ctype, i ) ) == companionName ) then
            return true
        end--if
    end--for
    return false
end--LibPetAndMountDatabase:IsCompanionType( companionName, ctype )

function LibPetAndMountDatabase:GetSummonedCompanionIndex( ctype )
    for i = 1, GetNumCompanions( ctype ) do
        if( select( 5, GetCompanionInfo( ctype, i ) ) ) then
            return i
        end--if
    end--for
    return nil
end--LibPetAndMountDatabase:GetSummonedCompanionIndex( ctype )

function LibPetAndMountDatabase:FindCompanion( ctype, spellID )
    if( spellID and ( spellID < 0 ) ) then
        spellID = spellID * -1
    end--if
    if( spellID ) then
        for i = 1, GetNumCompanions( ctype ) do
            if( select( 3, GetCompanionInfo( ctype, i ) ) == spellID ) then
                return i
            end--if
        end--for
    end--if
    return nil
end--LibPetAndMountDatabase:FindCompanion( ctype, spellID )
